
#include <magick/magick.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include "xmagick.h"

static void add_pixel(unsigned long pixel, unsigned long *pixels,
		int *npixels, int max_pixels)
{
	int i;

	if (*npixels >= max_pixels) return;
	for (i = 0; i < *npixels; i++) {
		if (pixel == pixels[i]) return;
	}
	pixels[i++] = pixel;
	*npixels = i;
}

static int npixels_in_ximage(XImage *img, int max_pixels)
{
	int x, y;
	int npixels = 0;
	unsigned long *pixels = malloc(max_pixels*sizeof(*pixels));
	for (y = 0; y < img->height; y++) {
		for (x = 0; x < img->width; x++) {
			unsigned long pixel = XGetPixel(img, x, y);
			add_pixel(pixel, pixels, &npixels, max_pixels);
		}
	}
	return npixels;
}

int main(int argc, char **argv)
{
	Image *image, *image2;
	ImageInfo image_info;
	XImage *ximage;
	Display *display;

	display = XOpenDisplay(NULL);
	if (display == NULL) {
		fprintf(stderr, "Can't open X display\n");
		exit(1);
	}

	GetImageInfo(&image_info);
	strcpy(image_info.filename, "image.xpm");
	image = ReadImage(&image_info);
	if (image == NULL) return 1;

	printf("%ld colors in image\n", GetNumberColors(image, NULL));
	ximage = XMagickImageToXImage(display, image);
	printf("%d pixels in ximage\n", npixels_in_ximage(ximage, 256));
	image2 = XMagickXImageToImage(display, ximage);
	printf("%ld colors in image2\n", GetNumberColors(image2, NULL));
	strcpy(image2->filename, "image2.xpm");

	/* Save to disk */
	WriteImage(&image_info, image2);

	/* Free resources */
	DestroyImage(image);
	DestroyImage(image2);
	XDestroyImage(ximage);
	XCloseDisplay(display);

	return 0;
}
